@@ -499,8 +499,7 @@ sub _augment {
 
 	warn("Config augment error: unknown item $path$key\n")
 	  unless exists $self->{$key}
-	    || $path eq "pdf.fontconfig."
-	    || $path =~ /^pdf\.(?:info|fonts)\./
+	    || $path =~ /^pdf\.(?:info|fonts|fontconfig)\./
 	    || $path =~ /^meta\./
 	    || $key =~ /^_/;
 
@@ -1005,6 +1004,9 @@ sub default_config() {
     "meta" : {
     },
 
+    // Assets.
+    "assets" : {},
+
     // Dates. Format is a strftime template.
     "dates" : {
         "today" : {
@@ -1076,7 +1078,6 @@ sub default_config() {
     ],
 
     // Printing chord diagrams.
-    // "auto": automatically add unknown chords as empty diagrams.
     // "show": prints the chords used in the song.
     //         "all": all chords used.
     //         "user": only prints user defined chords.
@@ -1086,7 +1087,6 @@ sub default_config() {
     // Note: The type of diagram (string or keyboard) is determined
     // by the value of "instrument.type".
     "diagrams" : {
-	"auto"     :  false,
 	"show"     :  "all",
 	"sorted"   :  false,
         "suppress" :  [],
