@@ -112,15 +112,16 @@ sub make_preprocessor {
 	my @targets;
 	my $code = "";
 	foreach ( @{ $prp->{$linetype} } ) {
+	    my $flags = $_->{flags} // "g";
 	    $code .= "m\0" . $_->{select} . "\0 && "
 	      if $_->{select};
 	    if ( $_->{pattern} ) {
 		$code .= "s\0" . $_->{pattern} . "\0"
-		  . $_->{replace} . "\0g;\n";
+		  . $_->{replace} . "\0$flags;\n";
 	    }
 	    else {
 		$code .= "s\0" . quotemeta($_->{target}) . "\0"
-		  . quotemeta($_->{replace}) . "\0g;\n";
+		  . quotemeta($_->{replace}) . "\0$flags;\n";
 	    }
 	}
 	if ( $code ) {
@@ -218,4 +219,19 @@ sub prp2cfg {
 
 push( @EXPORT, 'prp2cfg' );
 
+# Remove markup.
+sub demarkup {
+    my ( $t ) = @_;
+    return join( '', grep { ! /^\</ } splitmarkup($t) );
+}
+push( @EXPORT, 'demarkup' );
+
+# Split into markup/nonmarkup segments.
+sub splitmarkup {
+    my ( $t ) = @_;
+    my @t = split( qr;(</?(?:[-\w]+|span\s.*?)>);, $t );
+    return @t;
+}
+push( @EXPORT, 'splitmarkup' );
+
 1;
