@@ -4,6 +4,4 @@ use Text::Layout;
 use Text::Layout::FontConfig;
 use Text::Layout::FontDescriptor;
 use Text::Layout::PDFAPI2;
-use Text::Layout::PDFAPI2::ImageElement;
-use Text::Layout::ElementRole;
 use Text::Layout::Version;
